package com.jobs.jobhandler;

import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

@Component
public class MyTestJobs {

    // 注解使用方法
    // @XxlJob(value="自定义jobhandler名称",
    // init = "JobHandler初始化方法",
    // destroy = "JobHandler销毁方法")

    //----------------------------------------

    //该 demo 展示日志记录，以及返回任务执行状态，可以在 web 界面中查看
    @XxlJob("JobHandler01")
    public void testJob1() {
        System.out.println("JobHandler01 每 3 秒执行一次...");

        //通过 XxlJobHelper.log 记录的日志，可在 web 界面中进行查看
        XxlJobHelper.log("我自定义的日志："
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));

        //通过 XxlJobHelper.handleSuccess 方法，在 web 界面中会展示任务执行成功状态
        //默认情况下，任务执行都是成功状态，不需要刻意操作，
        //XxlJobHelper.handleSuccess("执行成功");

        //通过 XxlJobHelper.handleFail 方法，在 web 界面中会展示任务执行失败状态
        //如果任务执行过程中有异常，可以在 catch 方法中记录错误日志，让任务展示执行失败状态
        //XxlJobHelper.handleFail("错误日志");
    }

    //----------------------------------------

    //该 demo 展示如果获取 web 界面中配置的传入参数值
    @XxlJob("JobHandler02")
    public void testJob2() {
        //可以通过 XxlJobHelper.getJobParam() 方法获取 web 界面上配置的参数值
        //你可以传入 json 格式的参数值，方便在这里进行解析使用。
        String parm = XxlJobHelper.getJobParam();
        System.out.println("JobHandler02 每 5 秒执行一次，传入的 parm 参数为：" + parm);
    }

    //----------------------------------------

    //该 demo 展示 init 方法和 destory 方法的使用
    @XxlJob(value = "JobHandler03", init = "JobHandler03Init", destroy = "JobHandler03Destory")
    public void testJob3() {
        System.out.println("JobHandler03 每 2 秒执行一次，seq 为：" + seq);
        seq = seq + 1;
    }

    //定义一个变量
    private Integer seq;

    //自定义 init 方法，
    //当前的 Springboot 程序启动后，JobHandler03 第一次被调用之前执行，
    //该方法仅仅只会执行一次，除非重启了当前的 Springboot 程序
    public void JobHandler03Init() {
        seq = 0;
        System.out.println("everyFifteenJob 执行了初 init 方法...");
    }

    //当前的 spring 容器销毁时执行该方法，通常是 SprngBoot 停止的时候销毁 Spring 容器，
    //一般情况下我们看不到下面代码打印的日志，除非采用特殊的方案停止 spring 容器。
    public void JobHandler03Destory() {
        System.out.println("everyFifteenJob 执行了 destory 方法...");
    }

    //----------------------------------------

    //该 demo 演示如果获取分片信息
    @XxlJob("JobHandler04")
    public void testJobs4() {
        //获取该 springboot 程序的分片索引（索引从 0 开始）
        int shardIndex = XxlJobHelper.getShardIndex();
        //总分片数，springboot 执行器的实例总数
        int shardTotal = XxlJobHelper.getShardTotal();

        //获取所有的任务列表
        List<Integer> list = getList();
        //根据【列表索引】与【总分片数】的余数是否等于【分片索引】，确定当前 springboot 程序要执行的任务
        for (int i = 0; i < list.size(); i++) {
            if (i % shardTotal == shardIndex) {
                System.out.println("当前第" + shardIndex + "分片执行了，任务项为：" + list.get(i));
            }
        }
    }

    //表示 10000 个任务
    public List<Integer> getList() {
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i < 10000; i++) {
            list.add(i);
        }
        return list;
    }
}
